import { useMemo, useRef } from 'react';
import { useFirestoreDoc, useFirestoreDocOnce, } from 'reactfire';
import { useDocumentSnapData } from './useSnapData.js';
import { useFirestoreErrorLogger, useSafeRef } from './utils.js';
const createUseTypedDocHook = (hook) => {
    return (typedRef, dataOptions = {}) => {
        const { safeRef, refChanged } = useSafeRef(typedRef.raw);
        const memoizedTypedRef = useRef();
        if (refChanged) {
            memoizedTypedRef.current = typedRef;
        }
        const status = hook(safeRef, { suspense: true });
        useFirestoreErrorLogger(status.error);
        const { snap, data } = useDocumentSnapData(typedRef, status.data, dataOptions);
        return useMemo(() => {
            refChanged;
            return { ...status, typedRef: memoizedTypedRef.current, snap, data };
        }, [data, refChanged, snap, status]);
    };
};
export const useTypedDoc = createUseTypedDocHook(useFirestoreDoc);
export const useTypedDocOnce = createUseTypedDocHook(useFirestoreDocOnce);
